/* pluge.c -  Picture Lune Up Generator Equipment */
#include <u.h>
#include <libc.h>
#include <draw.h>
#include <event.h>

enum {
	Lines = 576,
	Pixels = 720,
	White = 255,
	Black = 0,
	Nstep = 8
};

void
usage(void)
{
	fprint(2, "usage: %s [-p w] [-l h]\n", argv0);
	fprint(2, "  -l h      image height in lines (%d)\n", Lines);
	fprint(2, "  -p w      image width in pixles (%d)\n", Pixels);
	exits("usage");
}

/*
 * The Pluge spec defines uses mV for vertial specs,
 * it expects white to be 700mV
 */
static int
mv2int(int mV)
{
	if(mV < 0)			/* negative brightness is hard */
		return 0;
	return (mV*White)/700+0.5;
}

/*
 * The Pluge spec uses microseconds for horizontal specs.
 * NB: assuming System-I:
 *		64usec lines, of which 4.7usec is blanking
 */
int
us2pix(int us, int pixels)
{
	return (((us-4.7)/64.0)*pixels)+0.5;
}
static void
fill(uchar *tmp, uchar mV, int pixels)
{
	int	p;

	memset(tmp, 0, pixels);
	if(mV == 0)
		return;

	for(p = us2pix(21.5, pixels); p < us2pix(23.8, pixels); p++)
		tmp[p] = mv2int(-14);				/* -14mV black bar */
	for(; p < us2pix(26.1, pixels); p++)
		tmp[p] = mv2int(14);				/* +14mV black bar */
	for(p = us2pix(40.0, pixels); p < us2pix(52.0, pixels); p++)
		tmp[p] = mv2int(mV);				/* vertical step wedge */
}

static void
sixth(int *l, uchar *buf, uchar *tmp, int linelen, int pixels, int lines)
{
	int i;
	uchar *bp;

	for(i = 0; i < lines/6; i++){
		bp = buf + (*l)++ * linelen;
		memcpy(bp, tmp, pixels);
	}
}

void
main(int argc, char *argv[])
{
	Rectangle r;
	char str[16];
	uchar *buf, *tmp;
	int size, p, l, linelen, pixels, lines;

	lines = Lines;
	pixels = Pixels;
	ARGBEGIN {
	case 'l':
		lines = atof(EARGF(usage()));
		break;
	case 'p':
		pixels = atof(EARGF(usage()));
		break;
	default:
		usage();
		break;
	}ARGEND;

	r.min = Pt(0, 0);
	r.max = Pt(pixels, lines);
	linelen = bytesperline(r, chantodepth(GREY8));
	size = linelen * Dy(r);
	if((buf = mallocz(size, 1)) == nil)
		sysfatal("no memory: %r\n");
	if((tmp = mallocz(pixels, 1)) == nil)
		sysfatal("no memory: %r\n");

	l = 0;
	fill(tmp, 0, pixels);
	sixth(&l, buf, tmp, linelen, pixels, lines);

	fill(tmp, 700, pixels);
	sixth(&l, buf, tmp, linelen, pixels, lines);

	fill(tmp, 450, pixels);
	sixth(&l, buf, tmp, linelen, pixels, lines);

	fill(tmp, 200, pixels);
	sixth(&l, buf, tmp, linelen, pixels, lines);

	fill(tmp, 110, pixels);
	sixth(&l, buf, tmp, linelen, pixels, lines);

	fill(tmp, 0, pixels);
	sixth(&l, buf, tmp, linelen, pixels, lines);

	chantostr(str, GREY8);
	fprint(1, "%11s %11d %11d %11d %11d ", str, r.min.x, r.min.y, r.max.x, r.max.y);
	if(write(1, buf, size) != size)
		sysfatal("write fail");
	exits(0);
}

